"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/*
 * Copyright (c) 2017 yvt
 *
 * This file is a part of hyper3d-envmapgen. Please read the license text that
 * comes with the source code for use conditions.
 */
var utils_1 = require("./utils");
var $fr = Math.fround || (function (x) { return x; });
var ImageFormat;
(function (ImageFormat) {
    ImageFormat[ImageFormat["Srgbx8"] = 0] = "Srgbx8";
    ImageFormat[ImageFormat["Srgba8StraightAlpha"] = 1] = "Srgba8StraightAlpha";
    ImageFormat[ImageFormat["RgbaF32PremulAlpha"] = 2] = "RgbaF32PremulAlpha";
})(ImageFormat = exports.ImageFormat || (exports.ImageFormat = {}));
function coerceImageToSrgba8StraightAlpha(image, width, height) {
    if (image instanceof Uint8Array) {
        return image;
    }
    if (!width || !height) {
        if (image instanceof HTMLImageElement) {
            width = image.naturalWidth;
            height = image.naturalHeight;
        }
        else if (image instanceof HTMLCanvasElement) {
            width = image.width;
            height = image.height;
        }
        else if (image instanceof HTMLVideoElement) {
            width = image.videoWidth;
            height = image.videoHeight;
        }
    }
    if (!width || !height) {
        throw new Error("Cannot guess the size of the image");
    }
    var canvas = document.createElement('canvas');
    canvas.width = width;
    canvas.height = height;
    var context = canvas.getContext('2d');
    context.drawImage(image, 0, 0, width, height);
    return context.getImageData(0, 0, width, height).data;
}
exports.coerceImageToSrgba8StraightAlpha = coerceImageToSrgba8StraightAlpha;
function coerceToRgbaF32PremulAlphaFrom(image, format, width, height) {
    switch (format) {
        case ImageFormat.RgbaF32PremulAlpha:
            if (image instanceof Float32Array) {
                return image;
            }
            else {
                throw new Error("Invalid type for RgbaF32PremulAlpha");
            }
        case ImageFormat.Srgba8StraightAlpha: {
            if (image instanceof Float32Array) {
                throw new Error("Invalid type for RgbaF32PremulAlpha");
            }
            if (!(image instanceof Uint8Array)) {
                image = coerceImageToSrgba8StraightAlpha(image, width, height);
            }
            var u8 = image;
            var f32 = new Float32Array(u8.length);
            var table_1 = exports.SRGB_DECODE_TABLE;
            for (var i = 0; i < u8.length; i += 4) {
                var a = u8[i + 3] * (1 / 255) + 1e-20;
                f32[i] = table_1[u8[i]] * a;
                f32[i + 1] = table_1[u8[i + 1]] * a;
                f32[i + 2] = table_1[u8[i + 2]] * a;
                f32[i + 3] = a;
            }
            return f32;
        }
        case ImageFormat.Srgbx8: {
            if (image instanceof Float32Array) {
                throw new Error("Invalid type for RgbaF32PremulAlpha");
            }
            if (!(image instanceof Uint8Array)) {
                image = coerceImageToSrgba8StraightAlpha(image, width, height);
            }
            var u8 = image;
            var f32 = new Float32Array(u8.length);
            var table_2 = exports.SRGB_DECODE_TABLE;
            for (var i = 0; i < u8.length; i += 4) {
                f32[i] = table_2[u8[i]];
                f32[i + 1] = table_2[u8[i + 1]];
                f32[i + 2] = table_2[u8[i + 2]];
                f32[i + 3] = 1;
            }
            return f32;
        }
        default:
            throw new Error("Invalid ImageFormat: " + format);
    }
}
exports.coerceToRgbaF32PremulAlphaFrom = coerceToRgbaF32PremulAlphaFrom;
function coerceRgbaF32PremulAlphaTo(image, format) {
    switch (format) {
        case ImageFormat.RgbaF32PremulAlpha:
            return image;
        case ImageFormat.Srgba8StraightAlpha:
        case ImageFormat.Srgbx8: {
            var u8 = new Uint8Array(image.length);
            var table_3 = exports.SRGB_ENCODE_TABLE;
            for (var i = 0; i < u8.length; i += 4) {
                var a = image[i + 3];
                var scale = $fr(65000 / a);
                u8[i] = table_3[$fr(image[i] * scale) & 0xffff];
                u8[i + 1] = table_3[$fr(image[i + 1] * scale) & 0xffff];
                u8[i + 2] = table_3[$fr(image[i + 2] * scale) & 0xffff];
                u8[i + 3] = $fr(a * 255) + 0.5;
            }
            return u8;
        }
        default:
            throw new Error("Invalid ImageFormat: " + format);
    }
}
exports.coerceRgbaF32PremulAlphaTo = coerceRgbaF32PremulAlphaTo;
exports.SRGB_DECODE_TABLE = new Float32Array(utils_1.table(256, function (i) {
    i /= 255;
    return (i < 0.04045 ? i / 12.92 : Math.pow(((i + 0.055) / 1.055), 2.4));
}));
exports.SRGB_ENCODE_TABLE = new Uint8Array(utils_1.table(65536, function (i) {
    i = Math.min(i / 65000, 1);
    return (i < 0.0031308 ? 12.92 * i : 1.055 * (Math.pow(i, (1 / 2.4))) - 0.055) * 255 + 0.5 | 0;
}));
function resampleRgbF32(inImage, inWidth, inHeight, outWidth, outHeight) {
    var outImage = new Float32Array(outWidth * outHeight * 4);
    if (inWidth < outWidth || inHeight < outHeight) {
        throw new Error("Does not support magnification yet");
    }
    var outIndex = 0;
    if (inWidth === outWidth * 2 && inHeight === outHeight * 2) {
        for (var y = 0; y < outHeight; ++y) {
            var inIndex1 = (y * 2 * inWidth) << 2;
            var inIndex2 = inIndex1 + (inWidth << 2);
            for (var x = 0; x < outWidth; ++x) {
                var r1 = inImage[inIndex1], g1 = inImage[inIndex1 + 1], b1 = inImage[inIndex1 + 2], a1 = inImage[inIndex1 + 3];
                var r2 = inImage[inIndex1 + 4], g2 = inImage[inIndex1 + 4 + 1], b2 = inImage[inIndex1 + 4 + 2], a2 = inImage[inIndex1 + 4 + 3];
                var r3 = inImage[inIndex2], g3 = inImage[inIndex2 + 1], b3 = inImage[inIndex2 + 2], a3 = inImage[inIndex2 + 3];
                var r4 = inImage[inIndex2 + 4], g4 = inImage[inIndex2 + 4 + 1], b4 = inImage[inIndex2 + 4 + 2], a4 = inImage[inIndex2 + 4 + 3];
                outImage[outIndex] = $fr($fr(r1 + r2) + $fr(r3 + r4)) * 0.25;
                outImage[outIndex + 1] = $fr($fr(g1 + g2) + $fr(g3 + g4)) * 0.25;
                outImage[outIndex + 2] = $fr($fr(b1 + b2) + $fr(b3 + b4)) * 0.25;
                outImage[outIndex + 3] = $fr($fr(a1 + a2) + $fr(a3 + a4)) * 0.25;
                outIndex += 4;
                inIndex1 += 8;
                inIndex2 += 8;
            }
        }
    }
    else {
        var inY = $fr(0.5 * inHeight / outHeight - 0.5);
        var inDY = $fr(inHeight / outHeight);
        var inX = $fr(0.5 * inWidth / outWidth - 0.5);
        var inDX = $fr(inWidth / outWidth);
        for (var y = 0; y < outHeight; ++y) {
            var inBase1 = Math.floor(inY) * inWidth << 2;
            var inBase2 = Math.ceil(inY) * inWidth << 2;
            var facY2 = $fr(inY - Math.floor(inY));
            var facY1 = $fr(1 - facY2);
            var inX2 = inX;
            for (var x = 0; x < outWidth; ++x) {
                // Linear interpolation
                var x1 = Math.floor(inX2) << 2;
                var x2 = Math.ceil(inX2) << 2;
                var facX2 = $fr(inX2 - Math.floor(inX2));
                var facX1 = $fr(1 - facX2);
                var r1 = inImage[inBase1 + x1], g1 = inImage[inBase1 + x1 + 1], b1 = inImage[inBase1 + x1 + 2], a1 = inImage[inBase1 + x1 + 3];
                var r2 = inImage[inBase1 + x2], g2 = inImage[inBase1 + x2 + 1], b2 = inImage[inBase1 + x2 + 2], a2 = inImage[inBase1 + x2 + 3];
                var r3 = inImage[inBase2 + x1], g3 = inImage[inBase2 + x1 + 1], b3 = inImage[inBase2 + x1 + 2], a3 = inImage[inBase2 + x1 + 3];
                var r4 = inImage[inBase2 + x2], g4 = inImage[inBase2 + x2 + 1], b4 = inImage[inBase2 + x2 + 2], a4 = inImage[inBase2 + x2 + 3];
                r1 = $fr($fr(r1 * facY1) + $fr(r3 * facY2));
                g1 = $fr($fr(g1 * facY1) + $fr(g3 * facY2));
                b1 = $fr($fr(b1 * facY1) + $fr(b3 * facY2));
                a1 = $fr($fr(a1 * facY1) + $fr(a3 * facY2));
                r2 = $fr($fr(r2 * facY1) + $fr(r4 * facY2));
                g2 = $fr($fr(g2 * facY1) + $fr(g4 * facY2));
                b2 = $fr($fr(b2 * facY1) + $fr(b4 * facY2));
                a2 = $fr($fr(a2 * facY1) + $fr(a4 * facY2));
                r1 = $fr($fr(r1 * facX1) + $fr(r2 * facX2));
                g1 = $fr($fr(g1 * facX1) + $fr(g2 * facX2));
                b1 = $fr($fr(b1 * facX1) + $fr(b2 * facX2));
                a1 = $fr($fr(a1 * facX1) + $fr(a2 * facX2));
                outImage[outIndex] = r1;
                outImage[outIndex + 1] = g1;
                outImage[outIndex + 2] = b1;
                outImage[outIndex + 3] = a1;
                outIndex += 4;
                inX2 += inDX;
                inX2 = $fr(inX2);
            }
            inY += inDY;
            inY = $fr(inY);
        }
    }
    return outImage;
}
exports.resampleRgbF32 = resampleRgbF32;
//# sourceMappingURL=image.js.map