/// <reference types="webassembly-js-api" />
import { ImageFormat } from './image';
export declare type Ptr = number;
/**
 * Options relevant to the creation of the core WebAssembly module.
 */
export interface CoreOptions {
    /**
     * An optional pre-compiled or cached WebAssembly module, created from
     * `hyperenvmap_wasm.wasm`.
     *
     * If omitted, a global module will be created automatically by calling
     * `getGlobalCoreModule` or `getGlobalCoreModulePromise`.
     */
    module?: WebAssembly.Module;
    /**
     * An optional pre-created or cached WebAssembly instance.
     */
    instance?: WebAssembly.Instance;
}
/** The API exported by `hyperenvmap_wasm.wasm`. */
export interface CoreExports {
    memory: WebAssembly.Memory;
    emg_init(): void;
    emg_malloc(size: number): Ptr;
    emg_free(ptr: Ptr): void;
    emg_ltasg_single(out_faces: Ptr, in_faces: Ptr, size: number, kernel: Ptr, kernel_size: number, kernel_scale: number, phase: number): void;
}
/**
 * Retrieves the built-in core WebAssembly binary module. Can be used to
 * customize the initialization behavior.
 */
export declare function getCoreWasmBlob(): Uint8Array;
/**
 * Retrieves the compiled global WebAssembly module. Triggers a synchronous
 * compilation on first use.
 */
export declare function getGlobalCoreModule(): WebAssembly.Module;
/**
 * Retrieves the compiled global WebAssembly module.
 */
export declare function getGlobalCoreModulePromise(): Promise<WebAssembly.Module>;
/**
 * Provides a JavaScript interface to the core services.
 */
export declare class CoreInstance implements Readonly<CoreOptions> {
    readonly module: WebAssembly.Module;
    readonly instance: WebAssembly.Instance;
    /**
     * Asynchronous constructor of `CoreInstance`.
     */
    static create(options?: Readonly<CoreOptions>): Promise<CoreInstance>;
    /**
     * Synchronous constructor of `CoreInstance`. `options.module` and
     * `options.instance` must not be `null` or `undefined` if this
     * constructor is called from the main thread.
     */
    constructor(options?: Readonly<CoreOptions>);
    /**
     * Applies a linear-time approximate spherical Gaussian blur on a given
     * cube map image.
     *
     * `C` is the number of elements per pixel and fixed at `4` in the current
     * version.
     *
     * @param inFaces The input cube map image. Must have at least 6 elements
     *                and each one must be at least `(size ** 2) * C` long.
     * @param outFaces The output cube map image. Must have at least 6 elements
     *                 and each one must be at least `(size ** 2) * C` long.
     *                 You can specfify the same value as `inFaces` for in-place
     *                 operation.
     * @param size The length of each side of the cube map, in pixels.
     * @param format The image format. Must be `RgbaF32PremulAlpha`.
     * @param kernel The convolution kernel. Must be odd-sized.
     * @param kernelScale The spatial scale of the kernel. See `ltasgblur.rs` for
     *                    more info.
     * @param numPasses Specifies how many times a Gaussian blur is applied on
     *                  the image. It is more efficient to use this parameter
     *                  than calling this function for multiple times.
     */
    ltasg(inFaces: ArrayLike<Float32Array>, outFaces: ArrayLike<Float32Array>, size: number, format: ImageFormat, kernel: Float32Array, kernelScale: number, numPasses: number): void;
}
